package com.jiuqi.db;

import com.jiuqi.util.StringArrayOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public final class DBTrace {
   private static Object syncObj = new Object();
   private static boolean doTrace = false;
   private static Map traceFinder = new HashMap();

   private DBTrace() {
   }

   public static void createConnection(Object key) {
      if (doTrace) {
         synchronized(syncObj) {
            TraceEntry ex = new TraceEntry(key);
            traceFinder.put(key, ex);
         }
      }
   }

   public static void closeConnection(Object key) {
      if (doTrace) {
         synchronized(syncObj) {
            traceFinder.remove(key);
         }
      }
   }

   public static void setTrace(boolean value) {
      if (doTrace != value) {
         if (value) {
            System.out.println("Strarted Trace DB Connection Objects");
            doTrace = true;
         } else {
            System.out.println("Stoped Trace DB Connection Objects");
            traceFinder.clear();
            doTrace = false;
         }

      }
   }

   public static List getTraceResult() {
      synchronized(syncObj) {
         List result = new ArrayList();
         Iterator it = traceFinder.keySet().iterator();

         while(it.hasNext()) {
            Object obj = traceFinder.get(it.next());
            if (obj != null) {
               result.add(obj);
            }
         }

         return result;
      }
   }

   public static List getTraceStrings() {
      List traces = getTraceResult();
      List result = new ArrayList();

      for(int i = 0; i < traces.size(); ++i) {
         StringArrayOutputStream os = new StringArrayOutputStream();
         PrintStream ps = new PrintStream(os);
         TraceEntry item = (TraceEntry)traces.get(i);
         item.printStackTrace(ps);
         result.add(os.toStringArray());
      }

      return result;
   }

   public static boolean inTracing() {
      return doTrace;
   }

   public static void startLog(int fileCount, int fileSize, String fileDir) {
      PrintStream ps = new PrintStream((OutputStream)null, true);
      ps.flush();
   }

   public static final class TraceEntry extends Exception {
      private final Calendar createTime = Calendar.getInstance();
      private final String createThread = Thread.currentThread().getName();
      private static final int OBJ_UNKNOWN = 0;
      private static final int OBJ_CONNECTION = 1;
      private static final int OBJ_STATEMENT = 2;
      private static final int OBJ_RESULTSET = 3;
      private static final String[] OBJ_NAMES = new String[]{"UNKNOWN", "CONNECTION", "STATEMENT", "RESULTSET"};
      private int objType = 0;

      public TraceEntry(Object obj) {
         if (obj instanceof Connection) {
            this.objType = 1;
         } else if (obj instanceof Statement) {
            this.objType = 2;
         } else if (obj instanceof ResultSet) {
            this.objType = 3;
         }

      }

      public Calendar getCreateTime() {
         return this.createTime;
      }

      public String getCreateThread() {
         return this.createThread;
      }

      public String toString() {
         DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss(SSS)");
         StringBuffer buffer = new StringBuffer(super.toString());
         buffer.append(" (").append(OBJ_NAMES[this.objType]).append(')').append(" created by [").append(this.createThread).append("], on ").append(df.format(this.createTime.getTime()));
         return buffer.toString();
      }
   }
}
